@@ -25,19 +25,36 @@ except ImportError:
 
 
 bruker_bridge_present = False
+bruker_bridge_initialized = False
 
 try:
     # Try to find and load Bruker's dll for data conversion
     import opentims_bruker_bridge as obb
-    for so_path in obb.get_so_paths():
-        try:
-            opentimspy_cpp.setup_bruker_so(so_path)
-            bruker_bridge_present = True
-            break
-        except RuntimeError:
-            pass
+    bruker_bridge_present = True
 except ImportError:
     pass
 
+if bruker_bridge_present:
+    # Try to initialize it
+    excuses = []
+    so_paths = obb.get_so_paths()
+    for so_path in so_paths:
+        try:
+            opentimspy_cpp.setup_bruker_so(so_path)
+            bruker_bridge_initialized = True
+            break
+        except RuntimeError as e:
+            excuses.append(e)
+    if not bruker_bridge_initialized:
+        errmsg = []
+        errmsg.append(f"Failed to initialize the Bruker binary library for conversion. {len(excuses)} attempts were made, and here are the reasons they failed:\n\n")
+        for so_path, excuse in zip(so_paths, excuses):
+            errmsg.append("Path " + so_path + " failed:")
+            errmsg.append(str(excuse))
+            errmsg.append("")
+        errmsg.append("")
+        errmsg.append("Please either fix one of the above errors, or uninstall opentims_bruker_bridge module.")
 
-__version__ = "1.0.6"
+        raise ImportError('\n'.join(errmsg))
+
+__version__ = "1.0.7"
