@@ -21,8 +21,9 @@ import pathlib
 
 import opentimspy
 
+from .sql import tables_names, table2dict
 
-all_columns = ('frame','scan','tof','intensity','mz','dt','rt')
+all_columns = ('frame','scan','tof','intensity','mz','inv_ion_mobility','retention_time')
 all_columns_dtype = (np.uint32,)*4 + (np.double,)*3
 
 def hash_frame(X,
@@ -44,19 +45,42 @@ class OpenTIMS:
             Args:
                 analysis_directory (str, unicode string): path to the folder containing 'analysis.tdf' and 'analysis.tdf_raw'.
         """
-        analysis_directory = pathlib.Path(analysis_directory)
-        assert analysis_directory.exists(), f"There is no such location: {analysis_directory}"
+        self.analysis_directory = pathlib.Path(analysis_directory)
         self.handle = opentimspy.opentimspy_cpp.TimsDataHandle(str(analysis_directory))
+        self.frames = self.table2dict("Frames")
+        # make sure it is all sorted by retention time / frame number
+        sort_order = np.argsort(self.frames["Id"])
+        for column in self.frames:
+            self.frames[column] = self.frames[column][sort_order]
+        self.GlobalMetadata = self.table2dict("GlobalMetadata")
+        self.GlobalMetadata = dict(zip(self.GlobalMetadata['Key'], self.GlobalMetadata['Value']))
 
-        self.min_frame = self.handle.min_frame_id()
-        self.max_frame = self.handle.max_frame_id()
-        self.rts = self.frame2rt(range(self.min_frame, self.max_frame+1))
+        self.min_frame = self.frames['Id'][0]
+        self.max_frame = self.frames['Id'][-1]
+        # self.ms_types = np.array([self.handle.get_frame(i).msms_type 
+                                  # for i in range(self.min_frame, self.max_frame+1)])
+        self.ms_types = self.frames['MsMsType']
+        self.ms1_frames = np.arange(self.min_frame, self.max_frame+1)[self.ms_types == 0]
+        self.frames_no = self.max_frame - self.min_frame+1
+        self._ms1_mask = np.zeros(self.frames_no, dtype=bool)
+        self._ms1_mask[self.ms1_frames-1] = True
+        self.min_scan = 1
+        self.max_scan = self.frames['NumScans'].max()
+        self.min_intensity = 0
+        self.max_intensity = self.frames['MaxIntensity'].max()
+        self.retention_times = self.frames['Time'] # it's in seconds!
+        self.min_retention_time = self.retention_times[0]
+        self.max_retention_time = self.retention_times[-1]
+        self.min_inv_ion_mobility = float(self.GlobalMetadata['OneOverK0AcqRangeLower'])
+        self.max_inv_ion_mobility = float(self.GlobalMetadata['OneOverK0AcqRangeUpper'])
+        self.min_mz = float(self.GlobalMetadata['MzAcqRangeLower'])
+        self.max_mz = float(self.GlobalMetadata['MzAcqRangeUpper'])
+        # self.min_frame = self.handle.min_frame_id()
+        # self.max_frame = self.handle.max_frame_id()
+        # self.retention_times = self.frame2retention_time(range(self.min_frame, self.max_frame+1))# it's in seconds!
         self.peaks_cnt = self.handle.no_peaks_total()
         self.all_columns = all_columns
         self.all_columns_dtypes = all_columns_dtype
-        self.ms_types = np.array([self.handle.get_frame(i).msms_type 
-                                  for i in range(self.min_frame, self.max_frame+1)])
-        self.ms1_frames = np.arange(self.min_frame, self.max_frame+1)[self.ms_types == 0]
 
     def __len__(self):
         return self.peaks_cnt
@@ -70,8 +94,13 @@ class OpenTIMS:
         if hasattr(self, 'handle'):
             del self.handle
 
+    def tables_names(self):
+        return tables_names(self.analysis_directory/"analysis.tdf")
 
-    def frame2rt(self, frames):
+    def table2dict(self, name):
+        return table2dict(self.analysis_directory/"analysis.tdf", name)
+
+    def frame2retention_time(self, frames):
         frames = np.r_[frames]
         assert frames.min() >= self.min_frame, f"Minimal frame {frames.min()} <= truly minimal {self.min_frame}."
         assert frames.max() <= self.max_frame, f"Maximal frame {frames.max()} <= truly maximal {self.max_frame}."
@@ -93,7 +122,7 @@ class OpenTIMS:
 
 
     def _get_empty_arrays(self, size, selected_columns=all_columns):
-        """Return a dictionary of empty numpy arrays to be filled with delicious data. Some are left empty and thus not filled."""
+        """Return a dictionary of empty numpy arrays to be filled with raw data. Some are left empty and thus not filled."""
 
         return {col: np.empty(shape=size if col in selected_columns else 0,
                               dtype=dtype) 
@@ -141,39 +170,50 @@ class OpenTIMS:
             yield self.query(fr, columns)
 
 
-    def rt_query(self, min_rt, max_rt, columns=("rt", "frame", "dt", "scan", "mz", "tof", "intensity")):
+    def rt_query(self,
+                 min_retention_time,
+                 max_retention_time,
+                 columns=all_columns):
         """Get data from a selection of frames based on retention times.
 
-        Get all frames corresponding to retention times in a set "[min_rt, max_rt)".
+        Get all frames corresponding to retention times in a set "[min_retention_time, max_retention_time)".
+        Retention time is in seconds.
 
 
         Args:
-            min_rt (float): Minimal retention time.
-            max_rt (float): Maximal retention time to choose.
+            min_retention_time (float): Minimal retention time (in seconds).
+            max_retention_time (float): Maximal retention time to choose (in seconds).
             columns (tuple): which columns to extract? Defaults to all possible columns.
 
         Returns:
             dict: column to numpy array mapping.
         """
-        min_frame, max_frame = np.searchsorted(self.rts, (min_rt, max_rt))+1 #TODO: check border conditions!!!
+        min_frame, max_frame = np.searchsorted(self.retention_times,
+                                               (min_retention_time,
+                                                max_retention_time)) + 1 #TODO: check border conditions!!!
         return self.query(slice(min_frame, max_frame), columns)
 
 
-    def rt_query_iter(self, min_rt, max_rt, columns=("rt", "frame", "dt", "scan", "mz", "tof", "intensity")):
+    def rt_query_iter(self,
+                      min_retention_time,
+                      max_retention_time,
+                      columns=all_columns):
         """Iterate data from a selection of frames based on retention times.
 
-        Get all frames corresponding to retention times in a set "[min_rt, max_rt)".
+        Get all frames corresponding to retention times in a set "[min_retention_time, max_retention_time)".
 
 
         Args:
-            min_rt (float): Minimal retention time.
-            max_rt (float): Maximal retention time to choose.
+            min_retention_time (float): Minimal retention time.
+            max_retention_time (float): Maximal retention time to choose.
             columns (tuple): which columns to extract? Defaults to all possible columns.
 
         Yields:
             dict: column to numpy array mapping.
         """
-        min_frame, max_frame = np.searchsorted(self.rts, (min_rt, max_rt))+1 #TODO: check border conditions!!!
+        min_frame, max_frame = np.searchsorted(self.retention_times,
+                                               (min_retention_time,
+                                                max_retention_time)) + 1 #TODO: check border conditions!!!
         yield from self.query_iter(range(min_frame, max_frame+1), columns)
 
 
@@ -303,10 +343,6 @@ class OpenTIMS:
         Returns:
             np.array: Total Ion Current values per each frame (sums of intensities for each frame).
         """
-        I = (d['intensity'].sum()
-             for d in self.query_iter(slice(self.min_frame,
-                                            self.max_frame+1),
-                                      columns=('intensity',)))
-        return np.fromiter(I,
-                           dtype=int,
-                           count=self.max_frame-self.min_frame+1)
+        res = np.empty(shape=self.max_frame, dtype=np.uint32)
+        self.handle.per_frame_TIC(res)
+        return res
